unit DiscountCalc;
//=============================================================================
// Unit:        DiscountCalc
//
// Project:     HADP 1999 -- Demonstrate Embedding Python in Delphi
//
// Author:      Tres Seaver, Palladion Software, 1999.
//
// History:     23 January 1999     Initial Version
//=============================================================================

interface

uses Windows
   , Messages
   , SysUtils
   , Classes
   , Graphics
   , Controls
   , Forms
   , Dialogs
   , ExtCtrls
   , StdCtrls            
   , ComCtrls
   , Db
   , DBCtrls
   , DBTables
   , PythonEngine
   , PythonGUIInputOutput
   ;

type

  TfrmDiscountCalc = class( TForm )

    tblCUSTOMER                     : TTable;
    tblCUSTOMERCustNo               : TFloatField;
    tblCUSTOMERCompany              : TStringField;
    tblCUSTOMERAddr1                : TStringField;
    tblCUSTOMERAddr2                : TStringField;
    tblCUSTOMERCity                 : TStringField;
    tblCUSTOMERState                : TStringField;
    tblCUSTOMERZip                  : TStringField;
    tblCUSTOMERCountry              : TStringField;
    tblCUSTOMERPhone                : TStringField;
    tblCUSTOMERFAX                  : TStringField;
    tblCUSTOMERTaxRate              : TFloatField;
    tblCUSTOMERContact              : TStringField;
    tblCUSTOMERLastInvoiceDate      : TDateTimeField;
    dtsCUSTOMER                     : TDataSource;

    tblORDERS                       : TTable;
    tblORDERSOrderNo                : TFloatField;
    tblORDERSCustNo                 : TFloatField;
    tblORDERSSaleDate               : TDateTimeField;
    tblORDERSShipDate               : TDateTimeField;
    tblORDERSEmpNo                  : TIntegerField;
    tblORDERSShipToContact          : TStringField;
    tblORDERSShipToAddr1            : TStringField;
    tblORDERSShipToAddr2            : TStringField;
    tblORDERSShipToCity             : TStringField;
    tblORDERSShipToState            : TStringField;
    tblORDERSShipToZip              : TStringField;
    tblORDERSShipToCountry          : TStringField;
    tblORDERSShipToPhone            : TStringField;
    tblORDERSShipVIA                : TStringField;
    tblORDERSPO                     : TStringField;
    tblORDERSTerms                  : TStringField;
    tblORDERSPaymentMethod          : TStringField;
    tblORDERSItemsTotal             : TCurrencyField;
    tblORDERSTaxRate                : TFloatField;
    tblORDERSFreight                : TCurrencyField;
    tblORDERSAmountPaid             : TCurrencyField;
    dtsORDERS                       : TDataSource;

    pyeEngine                       : TPythonEngine;
    pydvTotal                       : TPythonDelphiVar;
    pydvPayment                     : TPythonDelphiVar;
    pydvTerms                       : TPythonDelphiVar;
    pydvDiscount                    : TPythonDelphiVar;
    pgioGuiIO                       : TPythonGUIInputOutput;

    pnlCustomer                     : TPanel;
    lblCustomer                     : TLabel;
    txtCUSTOMERCustNo               : TDBText;
    txtCUSTOMERCompany              : TDBText;
    navCUSTOMER                     : TDBNavigator;

    pnlOrder                        : TPanel;
    lblOrder                        : TLabel;
    txtORDERSOrderNo                : TDBText;
    lblPayment                      : TLabel;
    txtORDERSPaymentMethod          : TDBText;
    lblTerms                        : TLabel;
    txtORDERSTerms                  : TDBText;
    lblOrderTotal                   : TLabel;
    txtORDERSItemsTotal             : TDBText;
    navORDERS                       : TDBNavigator;

    lblDiscountFormula              : TLabel;
    mmoDiscountFormula              : TMemo;
    btnComputeDiscount              : TButton;
    pnlDiscountPct                  : TPanel;
    pnlDiscountAmount               : TPanel;
    lblPythonDebugOutput            : TLabel;
    rchPyDebugOutput                : TRichEdit;

    procedure GetInputValues( Sender    : TObject
                            ; var Data  : Variant
                            );

    procedure ComputeDiscount( Sender   : TObject
                             );

    procedure ShowDiscount(    Sender   : TObject
                          ;    Data     : Variant
                          );

  end;

var
  frmDiscountCalc: TfrmDiscountCalc;

implementation

{$R *.DFM}

//-----------------------------------------------------------------------------
// Method:      GetInputValues
//
// Class:       TfrmDiscountCalc
//
// Purpose:     Supply "Delphi" values to Python variables on demand.
//-----------------------------------------------------------------------------
procedure TfrmDiscountCalc.GetInputValues( Sender: TObject
                                         ; var Data: Variant
                                         );
begin
    with Sender as TPythonDelphiVar do
    begin
        //
        //  XXX:    Cheap hack on Tag to avoid writing more than one method.
        //
        case Tag of
            0 : Data := tblORDERSItemsTotal.AsCurrency;
            1 : Data := tblORDERSPaymentMethod.AsString;
            2 : Data := tblORDERSTerms.AsString;
            else
                Data := '';
        end;
    end;
end;


//-----------------------------------------------------------------------------
// Method:      ComputeDiscount
//
// Class:       TfrmDiscountCalc
//
// Purpose:     Let Python do its thing.
//-----------------------------------------------------------------------------
procedure TfrmDiscountCalc.ComputeDiscount( Sender: TObject );
begin
    pyeEngine.ExecStrings( mmoDiscountFormula.Lines );
end;


//-----------------------------------------------------------------------------
// Method:      ShowDiscount
//
// Class:       TfrmDiscountCalc
//
// Purpose:     Display the result of Python's labors, both as a percentage
//              discount and as a net.
//-----------------------------------------------------------------------------
procedure TfrmDiscountCalc.ShowDiscount( Sender : TObject
                                       ; Data   : Variant
                                       );
var
    discountPct : double;
    discount    : Currency;

begin
    discountPct                 := Data;
    pnlDiscountPct.Caption      := Format( '%g %%', [ discountPct ] );
    discount                    := tblORDERSItemsTotal.AsCurrency
                                 * ( 1.0 - ( discountPct / 100.0 ) );
    pnlDiscountAmount.Caption   := Format( '$%10.2f', [ discount ] );
end;

end.
